/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package nl.cloudfarming.client.sensor;

import java.awt.Dialog;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.geoviewer.LayerAction;
import nl.cloudfarming.client.geoviewer.Palette;
import nl.cloudfarming.client.model.SensorObservation;
import org.jdesktop.swingx.JXPanel;
import org.openide.DialogDescriptor;
import org.openide.DialogDisplayer;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;

/**
 * Action to change the palette of the sensor data 
 * @author Merijn Zengers
 */
@NbBundle.Messages("action.change_palette=Change palette")
public final class ChangePaletteAction extends LayerAction{

    private Dialog dlg;
    private ArrayList options = new ArrayList();
    private JComboBox comboBox;
    private SensorDataObject sensorDataObject;
    
    

    public ChangePaletteAction() {
        putValue(NAME, Bundle.action_change_palette());
        
        sensorDataObject = Utilities.actionsGlobalContext().lookup(SensorDataObject.class);
        options = (ArrayList) getPaletteOptions();
        comboBox = new JComboBox(options.toArray());
        comboBox.setEditable(false); 
    }

    /**
     * Gets all the options for a pallete.
     * Will lookup all the Palette ServiceProviders
     * adds all the options of Palette to the returning ArrayList
     * @return the List with palette options
     */
    private List<SensorObservation> getPaletteOptions() {
        assert sensorDataObject != null;
        
        ArrayList paletteOptions = new ArrayList();
        Collection<? extends Palette> palettes = Lookup.getDefault().lookupAll(Palette.class);
        //The Object present in the lookup should be the selected object
        for (Palette palette : palettes) {
            if (palette.supports(sensorDataObject.getClass())) {
                if (palette.isInterchangeable()) {
                    paletteOptions.addAll(palette.getInterchangeablePalettes());
                } else {
                    paletteOptions.add(palette);
                }
            }
        }
        return paletteOptions;
    }

    @Override
    public void actionPerformed(ActionEvent arg0) {
        assert sensorDataObject != null;

        JXPanel panel = new JXPanel(new MigLayout("wrap 2"));

        comboBox.setSelectedItem(sensorDataObject.getPalette());
        panel.add(new JLabel("choose:"));
        panel.add(comboBox);

        DialogDescriptor dd = new DialogDescriptor(panel, Bundle.action_change_palette());
        dd.setOptions(new Object[]{getOkButton(), getCancelButton()});
        dlg = DialogDisplayer.getDefault().createDialog(dd);
        dlg.setVisible(true);
    }

    private JButton getOkButton() {
        JButton jButtonOK = new JButton();
        jButtonOK.setText("Ok");
        jButtonOK.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent ae) {
                dlg.setVisible(false);
                if (comboBox.getSelectedItem() != null) {
                    sensorDataObject.setPalette((SensorPalette) comboBox.getSelectedItem());
                }

            }
        });
        return jButtonOK;
    }

    private JButton getCancelButton() {
        JButton jButtonCancel = new JButton();
        jButtonCancel.setText("cancel");
        jButtonCancel.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent ae) {
                dlg.setVisible(false);
            }
        });
        return jButtonCancel;
    }
}
